@@ -1,32 +1,7 @@
 #
-# [The "BSD license"]
-#  Copyright (c) 2012 Terence Parr
-#  Copyright (c) 2012 Sam Harwell
-#  Copyright (c) 2014 Eric Vergnaud
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#  3. The name of the author may not be used to endorse or promote products
-#     derived from this software without specific prior written permission.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
-#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-#  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
-#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
-#  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+# Use of this file is governed by the BSD 3-clause license that
+# can be found in the LICENSE.txt file in the project root.
 #/
 from io import StringIO
 from antlr4.RuleContext import RuleContext
@@ -70,7 +45,7 @@ class PredictionContext(object):
     #  }
     # </pre>
     #/
-    
+
     def __init__(self, cachedHashCode:int):
         self.cachedHashCode = cachedHashCode
 
@@ -85,7 +60,7 @@ class PredictionContext(object):
         return self.getReturnState(len(self) - 1) == self.EMPTY_RETURN_STATE
 
     def getReturnState(self, index:int):
-        raise "illegal!"
+        raise IllegalStateException("illegal!")
 
     def __hash__(self):
         return self.cachedHashCode
@@ -95,7 +70,7 @@ def calculateHashCode(parent:PredictionContext, returnState:int):
 
 def calculateListsHashCode(parents:[], returnStates:[] ):
     h = 0
-    for parent, returnState in parents, returnStates:
+    for parent, returnState in zip(parents, returnStates):
         h = hash((h, calculateHashCode(parent, returnState)))
     return h
 
@@ -242,7 +217,7 @@ class ArrayPredictionContext(PredictionContext):
                 if self.returnStates[i]==PredictionContext.EMPTY_RETURN_STATE:
                     buf.write("$")
                     continue
-                buf.write(self.returnStates[i])
+                buf.write(str(self.returnStates[i]))
                 if self.parents[i] is not None:
                     buf.write(' ')
                     buf.write(str(self.parents[i]))
@@ -251,6 +226,10 @@ class ArrayPredictionContext(PredictionContext):
             buf.write("]")
             return buf.getvalue()
 
+    def __hash__(self):
+        return self.cachedHashCode
+
+
 
 #  Convert a {@link RuleContext} tree to a {@link PredictionContext} graph.
 #  Return {@link #EMPTY} if {@code outerContext} is empty or null.
@@ -325,18 +304,18 @@ def merge(a:PredictionContext, b:PredictionContext, rootIsWildcard:bool, mergeCa
 #/
 def mergeSingletons(a:SingletonPredictionContext, b:SingletonPredictionContext, rootIsWildcard:bool, mergeCache:dict):
     if mergeCache is not None:
-        previous = mergeCache.get(a,b)
+        previous = mergeCache.get((a,b), None)
         if previous is not None:
             return previous
-        previous = mergeCache.get(b,a)
+        previous = mergeCache.get((b,a), None)
         if previous is not None:
             return previous
 
-    rootMerge = mergeRoot(a, b, rootIsWildcard)
-    if rootMerge is not None:
+    merged = mergeRoot(a, b, rootIsWildcard)
+    if merged is not None:
         if mergeCache is not None:
-            mergeCache.put(a, b, rootMerge)
-        return rootMerge
+            mergeCache[(a, b)] = merged
+        return merged
 
     if a.returnState==b.returnState:
         parent = merge(a.parentCtx, b.parentCtx, rootIsWildcard, mergeCache)
@@ -349,10 +328,10 @@ def mergeSingletons(a:SingletonPredictionContext, b:SingletonPredictionContext,
         # merge parents x and y, giving array node with x,y then remainders
         # of those graphs.  dup a, a' points at merged array
         # new joined parent so create new singleton pointing to it, a'
-        a_ = SingletonPredictionContext.create(parent, a.returnState)
+        merged = SingletonPredictionContext.create(parent, a.returnState)
         if mergeCache is not None:
-            mergeCache.put(a, b, a_)
-        return a_
+            mergeCache[(a, b)] = merged
+        return merged
     else: # a != b payloads differ
         # see if we can collapse parents due to $+x parents if local ctx
         singleParent = None
@@ -362,26 +341,24 @@ def mergeSingletons(a:SingletonPredictionContext, b:SingletonPredictionContext,
             # sort payloads and use same parent
             payloads = [ a.returnState, b.returnState ]
             if a.returnState > b.returnState:
-                payloads[0] = b.returnState
-                payloads[1] = a.returnState
+                payloads = [ b.returnState, a.returnState ]
             parents = [singleParent, singleParent]
-            a_ = ArrayPredictionContext(parents, payloads)
+            merged = ArrayPredictionContext(parents, payloads)
             if mergeCache is not None:
-                mergeCache.put(a, b, a_)
-            return a_
+                mergeCache[(a, b)] = merged
+            return merged
         # parents differ and can't merge them. Just pack together
         # into array; can't merge.
         # ax + by = [ax,by]
         payloads = [ a.returnState, b.returnState ]
         parents = [ a.parentCtx, b.parentCtx ]
         if a.returnState > b.returnState: # sort by payload
-            payloads[0] = b.returnState
-            payloads[1] = a.returnState
+            payloads = [ b.returnState, a.returnState ]
             parents = [ b.parentCtx, a.parentCtx ]
-        a_ = ArrayPredictionContext(parents, payloads)
+        merged = ArrayPredictionContext(parents, payloads)
         if mergeCache is not None:
-            mergeCache.put(a, b, a_)
-        return a_
+            mergeCache[(a, b)] = merged
+        return merged
 
 
 #
@@ -463,10 +440,10 @@ def mergeRoot(a:SingletonPredictionContext, b:SingletonPredictionContext, rootIs
 #/
 def mergeArrays(a:ArrayPredictionContext, b:ArrayPredictionContext, rootIsWildcard:bool, mergeCache:dict):
     if mergeCache is not None:
-        previous = mergeCache.get(a,b)
+        previous = mergeCache.get((a,b), None)
         if previous is not None:
             return previous
-        previous = mergeCache.get(b,a)
+        previous = mergeCache.get((b,a), None)
         if previous is not None:
             return previous
 
@@ -475,8 +452,8 @@ def mergeArrays(a:ArrayPredictionContext, b:ArrayPredictionContext, rootIsWildca
     j = 0 # walks b
     k = 0 # walks target M array
 
-    mergedReturnStates = [] * (len(a.returnState) + len( b.returnStates))
-    mergedParents = [] * len(mergedReturnStates)
+    mergedReturnStates = [None] * (len(a.returnStates) + len( b.returnStates))
+    mergedParents = [None] * len(mergedReturnStates)
     # walk and merge to yield mergedParents, mergedReturnStates
     while i<len(a.returnStates) and j<len(b.returnStates):
         a_parent = a.parents[i]
@@ -522,30 +499,30 @@ def mergeArrays(a:ArrayPredictionContext, b:ArrayPredictionContext, rootIsWildca
     # trim merged if we combined a few that had same stack tops
     if k < len(mergedParents): # write index < last position; trim
         if k == 1: # for just one merged element, return singleton top
-            a_ = SingletonPredictionContext.create(mergedParents[0], mergedReturnStates[0])
+            merged = SingletonPredictionContext.create(mergedParents[0], mergedReturnStates[0])
             if mergeCache is not None:
-                mergeCache.put(a,b,a_)
-            return a_
+                mergeCache[(a,b)] = merged
+            return merged
         mergedParents = mergedParents[0:k]
         mergedReturnStates = mergedReturnStates[0:k]
 
-    M = ArrayPredictionContext(mergedParents, mergedReturnStates)
+    merged = ArrayPredictionContext(mergedParents, mergedReturnStates)
 
     # if we created same array as a or b, return that instead
     # TODO: track whether this is possible above during merge sort for speed
-    if M==a:
+    if merged==a:
         if mergeCache is not None:
-            mergeCache.put(a,b,a)
+            mergeCache[(a,b)] = a
         return a
-    if M==b:
+    if merged==b:
         if mergeCache is not None:
-            mergeCache.put(a,b,b)
+            mergeCache[(a,b)] = b
         return b
     combineCommonParents(mergedParents)
 
     if mergeCache is not None:
-        mergeCache.put(a,b,M)
-    return M
+        mergeCache[(a,b)] = merged
+    return merged
 
 
 #
@@ -579,15 +556,14 @@ def getCachedPredictionContext(context:PredictionContext, contextCache:Predictio
         parent = getCachedPredictionContext(context.getParent(i), contextCache, visited)
         if changed or parent is not context.getParent(i):
             if not changed:
-                parents = [None] * len(context)
-                for j in range(0, len(context)):
-                    parents[j] = context.getParent(j)
+                parents = [context.getParent(j) for j in range(len(context))]
                 changed = True
             parents[i] = parent
     if not changed:
         contextCache.add(context)
         visited[context] = context
         return context
+
     updated = None
     if len(parents) == 0:
         updated = PredictionContext.EMPTY
