@@ -1,31 +1,6 @@
-# [The "BSD license"]
-#  Copyright (c) 2013 Terence Parr
-#  Copyright (c) 2013 Sam Harwell
-#  Copyright (c) 2014 Eric Vergnaud
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#  3. The name of the author may not be used to endorse or promote products
-#     derived from this software without specific prior written permission.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
-#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-#  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
-#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
-#  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
-#  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+# Use of this file is governed by the BSD 3-clause license that
+# can be found in the LICENSE.txt file in the project root.
 #/
 from uuid import UUID
 from io import StringIO
@@ -164,8 +139,8 @@ class ATNDeserializer (object):
         for i in range(0, numPrecedenceStates):
             stateNumber = self.readInt()
             atn.states[stateNumber].isPrecedenceRule = True
-            
-    def readRules(self, atn:ATN):        
+
+    def readRules(self, atn:ATN):
         nrules = self.readInt()
         if atn.grammarType == ATNType.LEXER:
             atn.ruleToTokenType = [0] * nrules
@@ -368,7 +343,7 @@ class ATNDeserializer (object):
 
     #
     # Analyze the {@link StarLoopEntryState} states in the specified ATN to set
-    # the {@link StarLoopEntryState#precedenceRuleDecision} field to the
+    # the {@link StarLoopEntryState#isPrecedenceDecision} field to the
     # correct value.
     #
     # @param atn The ATN.
@@ -387,7 +362,7 @@ class ATNDeserializer (object):
                 if isinstance(maybeLoopEndState, LoopEndState):
                     if maybeLoopEndState.epsilonOnlyTransitions and \
                             isinstance(maybeLoopEndState.transitions[0].target, RuleStopState):
-                        state.precedenceRuleDecision = True
+                        state.isPrecedenceDecision = True
 
     def verifyATN(self, atn:ATN):
         if not self.deserializationOptions.verifyATN:
