@@ -1,3 +1,8 @@
+#
+# Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+# Use of this file is governed by the BSD 3-clause license that
+# can be found in the LICENSE.txt file in the project root.
+#
 
 #
 # Represent a subset of XPath XML path syntax for use in identifying nodes in
@@ -42,20 +47,19 @@
 # <p>
 # Whitespace is not allowed.</p>
 #
-from xmlrpc.client import boolean
-from antlr4 import CommonTokenStream, DFA, PredictionContextCache, Lexer, LexerATNSimulator, ParserRuleContext, TerminalNode
+from antlr4 import CommonTokenStream, DFA, PredictionContextCache, Lexer, LexerATNSimulator
 from antlr4.InputStream import InputStream
 from antlr4.Parser import Parser
 from antlr4.RuleContext import RuleContext
 from antlr4.Token import Token
+from antlr4.atn.ATNDeserializer import ATNDeserializer
 from antlr4.error.ErrorListener import ErrorListener
 from antlr4.error.Errors import LexerNoViableAltException
 from antlr4.tree.Tree import ParseTree
 from antlr4.tree.Trees import Trees
-from antlr4.atn.ATNDeserializer import ATNDeserializer
-
 from io import StringIO
 
+
 def serializedATN():
     with StringIO() as buf:
         buf.write("\3\u0430\ud6d1\u8206\uad2d\u4417\uaef1\u8d80\uaadd\2\n")
@@ -115,7 +119,7 @@ class XPathLexer(Lexer):
 
     def __init__(self, input=None):
         super().__init__(input)
-        self.checkVersion("4.5")
+        self.checkVersion("4.6")
         self._interp = LexerATNSimulator(self, self.atn, self.decisionsToDFA, PredictionContextCache())
         self._actions = None
         self._predicates = None
@@ -290,12 +294,8 @@ class XPathRuleElement(XPathElement):
 
     def evaluate(self, t:ParseTree):
         # return all children of t that match nodeName
-        nodes = []
-        for c in Trees.getChildren(t):
-            if isinstance(c, ParserRuleContext ):
-                if (c.ruleIndex == self.ruleIndex ) == (not self.invert):
-                    nodes.append(c)
-        return nodes
+        return [c for c in Trees.getChildren(t) if isinstance(c, ParserRuleContext) and (c.ruleIndex == self.ruleIndex) == (not self.invert)]
+
 
 class XPathTokenAnywhereElement(XPathElement):
 
@@ -315,12 +315,8 @@ class XPathTokenElement(XPathElement):
 
     def evaluate(self, t:ParseTree):
         # return all children of t that match nodeName
-        nodes = []
-        for c in Trees.getChildren(t):
-            if isinstance(c, TerminalNode):
-                if (c.symbol.type == self.tokenType ) == (not self.invert):
-                    nodes.append(c)
-        return nodes
+        return [c for c in Trees.getChildren(t) if isinstance(c, TerminalNode) and (c.symbol.type == self.tokenType) == (not self.invert)]
+
 
 class XPathWildcardAnywhereElement(XPathElement):
 
